/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2021.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.syn.controllers.journal

import com.persagy.database.SFilter
import com.persagy.server.datacenter.models.entities.dictnew.DefClass
import com.persagy.server.datacenter.models.entities.objects.Project
import com.persagy.server.datacenter.models.jms.SynchronousMessage
import com.persagy.server.syn.services.BaseDataService
import com.persagy.server.syn.services.journal.SynchronousMessageService
import com.persagy.service.models.requests.SQueryRequest
import com.persagy.service.models.requests.SUpdateRequest
import com.persagy.service.models.responses.SBaseResponse
import com.persagy.service.models.responses.SQueryResponse
import io.swagger.v3.oas.annotations.Operation
import io.swagger.v3.oas.annotations.tags.Tag
import org.springframework.web.bind.annotation.*

/**
 * 同步日志
 *
 * @author wx  <zhangweixin@sagacloud.com>
 * @date  2021/2/25 10:32
 */
@Tag(name = "日志-同步日志")
@RestController
@RequestMapping("/record")
class JournalController {


    /**
     * 查询同步日志
     */
    @Operation(summary = "日志查询", description = "")
    @PostMapping(value = ["/query"])
    fun query(@RequestBody request: SQueryRequest): SQueryResponse<SynchronousMessage> {
        return SynchronousMessageService.pageQuery(request)
    }

    /**
     * 查询项目下对应的对象类型（设备、系统、空间）
     */
    @Operation(summary = "查询项目下对应的对象类型（设备、系统、空间）", description = "")
    @GetMapping(value = ["/query/type"])
    fun queryClassCode(@RequestParam objType: String,@RequestParam projectId: String): SQueryResponse<DefClass>{
        return SynchronousMessageService.typeQuery(objType,projectId)
    }

    /**
     * 更新日志信息
     *
     * @param   request        更新的内容对象
     * @return                  更新的结果
     */
    @Operation(summary = "更新日志信息", description = "忽略（sign = 3）")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: SUpdateRequest<SynchronousMessage>): SBaseResponse {
        return SynchronousMessageService.updateList(request)
    }

    /**
     * 重新同步消息
     */
    @Operation(summary = "重新同步消息", description = "")
    @PostMapping(value = ["/syn/data"])
    fun synData(@RequestBody request: SUpdateRequest<SynchronousMessage>): SBaseResponse {
        return SynchronousMessageService.synData(request)
    }

    @Operation(summary = "中台是否在线", description = "groupCode 集团编码 ")
    @GetMapping(value = ["/query/online"])
    fun isOnline(@RequestParam groupCode: String,@RequestParam projectId: String): SBaseResponse{
        return SynchronousMessageService.isOnline(groupCode,projectId)
    }

    @Operation(summary = "查询项目信息", description = "groupCode 集团编码")
    @GetMapping(value = ["/query/project"])
    fun queryProject(@RequestParam projectId: String): SQueryResponse<Project>{
        val request = SQueryRequest()
        return BaseDataService.projectService.pageQuery(request, arrayListOf(SFilter.eq("id",projectId!!)))
    }





}