/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2009-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

package com.persagy.server.service.dict

import com.persagy.database.SFilter
import com.persagy.mybatis.SMybatisDao
import com.persagy.server.Opt
import com.persagy.server.datacenter.models.entities.dictnew.DefClass
import com.persagy.server.datacenter.models.entities.dictnew.DefFuncId
import com.persagy.service.SObjectService
import com.persagy.service.models.enums.SResponseType
import com.persagy.service.models.responses.SQueryResponse

/**
 * 字典服务
 *
 * @author wx  <zhangweixin@sagacloud.com>
 * @date  2020/11/17 17:28
 */
object DictDefClassService: SObjectService<DefFuncId>(SMybatisDao(DefFuncId::class.java)) {

    /** 平台集团对象类型 */
    private val defClassService = SObjectService(SMybatisDao(DefClass::class.java))
    /**
     * 查询 M 和 L 类型的信息点
     */
    fun queryMOrL(dictTypeList: ArrayList<String>): SQueryResponse<DefFuncId> {
        /** 返回对象 */
        val sQueryResponse = SQueryResponse<DefFuncId>(SResponseType.success)
        /** 对象列表 */
        var list = ArrayList<DefFuncId>()
        /** 入参不为空 */
        if (!dictTypeList.isNullOrEmpty()){
            /** 遍历入参列表 */
            for (dictType in dictTypeList){
                /** 类型筛选 */
                when(dictType){
                    "project" -> {
                        /** 项目 */
                        list.add(typeStatisticsOne("project","项目"))
                    }
                    "building" -> {
                        /** 建筑 */
                        list.add(typeStatisticsOne("building","建筑"))
                    }
                    "floor" -> {
                        /** 楼层 */
                        list.add(typeStatisticsOne("floor","楼层"))
                    }
                    "equipment" -> {
                        /** 设备 */
                        list.add(typeStatisticsTwo("equipment","设备"))
                    }
                    "component" -> {
                        /** 部件 */
                        list.add(typeStatisticsTwo("component","部件"))
                    }
                    "system" -> {
                        /** 系统 */
                        list.add(typeStatisticsTwo("system","系统"))
                    }
                    "space" -> {
                        /** 空间 */
                        list.add(typeStatisticsTwo("space","空间"))
                    }
                }
            }

            /** 列表赋值 */
            sQueryResponse.content = list
        }else {
            /** 返回状态 */
            sQueryResponse.result = SResponseType.failure
        }

        /** 返回对象 */
        return sQueryResponse
    }

    /**
     * 项目、建筑、楼层 M 或 L 信息点统计
     *
     * @param code  编码
     * @param name  名称
     * @return 信息点对象
     */
    private fun typeStatisticsOne(code: String,name:String): DefFuncId {
        /** 实例化对象 */
        val defFuncId = DefFuncId()
        /** 查询 M 或 L 信息点 */
        val infosList = select(
            SFilter.eq("projectId", Opt.projectId!!), SFilter.eq("classCode", code),
           SFilter.not(SFilter.eq("category","STATIC"))
        ).exec()
        /** 编码赋值 */
        defFuncId.code = code
        /** 名称赋值 */
        defFuncId.name = name
        /** 列表赋值 */
        defFuncId.content = infosList
        /** 返回对象 */
        return defFuncId
    }

    /**
     * 设备、部件、系统、空间 M 或 L 信息点统计
     *
     * @param code  编码
     * @param name  名称
     * @return 信息点对象
     */
    private fun typeStatisticsTwo(code: String,name:String): DefFuncId {
        /** 返回对象 */
        val defFuncId = DefFuncId()
        /** 名称 */
        defFuncId.name = name
        /** 编码 */
        defFuncId.code = code
        /** 对象列表 */
        var equipList = ArrayList<DefFuncId>()
        /** 查询  */
        val typeList = defClassService.select(
            SFilter.eq("parentCode", code),
            SFilter.eq("projectId", Opt.projectId!!)
        ).exec()
        /** 遍历列表 */
        for (type in typeList){
            /** 实例化对象 */
            val defFuncId = DefFuncId()
            /** 名称 */
            defFuncId.name = type.name
            /** 编码 */
            defFuncId.code = type.code
            /** 查询 */
            val equipInfosList = select(
                SFilter.eq("projectId", Opt.projectId!!), SFilter.eq("classCode", type.code!!),
                SFilter.not(SFilter.eq("category","STATIC"))
            ).exec()
            /** 列表赋值 */
            defFuncId.content = equipInfosList
            /** 添加到列表 */
            equipList.add(defFuncId)
        }

        /** 列表赋值 */
        defFuncId.content = equipList
        /** 返回对象 */
        return defFuncId
    }



}